/**************************************************************************//**
 * @file
 * @brief VCMP - Voltage Comparator prototypes and definitions
 * @author shondll
 * @version 1.0.0
 ******************************************************************************
 *
 *****************************************************************************/

#include <stddef.h>
#include "efm32.h"
#include "vcmp.h"

/*************************************************************************
 * Function Name: VCMP_GetResult
 * Parameters: none
 *
 * Return:	0 - still in progress
 *			1 - Vdd above set level
 *			-1 - Vdd below set level
 *
 * Description: returns Vdd voltage level compared to the set reference
 *
 *************************************************************************/
int VCMP_GetResult(void)
{
	VCMP_TypeDef *vcmp = VCMP;
	
	if( !(vcmp->STATUS & VCMP_STATUS_VCMPACT) )
		return 0;
	else
		return (vcmp->STATUS & VCMP_STATUS_VCMPOUT) ? 1 : -1;
}

/*************************************************************************
 * Function Name: VCMP_SetLevel
 * Parameters: level - reference to the comparator (0..63)
 * Select VDD trigger level. Vtrig = 1.667V + (0.034V * level)
 *
 * Return: none
 *
 * Description: Sets new reference level for the comparator
 *
 *************************************************************************/
void VCMP_SetLevel(int level)
{
	VCMP_TypeDef *vcmp = VCMP;
	
	vcmp->CTRL &= ~_VCMP_CTRL_EN_MASK; // disable module
	while(vcmp->STATUS & VCMP_STATUS_VCMPACT) // wait for disabled state
		;
	
	vcmp->INPUTSEL &= ~_VCMP_INPUTSEL_TRIGLEVEL_MASK;
	vcmp->INPUTSEL |= level & _VCMP_INPUTSEL_TRIGLEVEL_MASK;
	
	vcmp->CTRL |= VCMP_CTRL_EN;
}

/*************************************************************************
 * Function Name: VCMP_GetLevel
 * Parameters: none
 *
 * Return: none
 * Select VDD trigger level. Vtrig = 1.667V + (0.034V * level)
 *
 * Description: Gets current reference level for the comparator
 *
 *************************************************************************/
int VCMP_GetLevel(void)
{
	int level;
	VCMP_TypeDef *vcmp = VCMP;
	
	level = vcmp->INPUTSEL & _VCMP_INPUTSEL_TRIGLEVEL_MASK;
	
	return level;
}

/*************************************************************************
 * Function Name: VCMP_Init
 * Parameters: none
 *
 * Return: none
 *
 * Description: initializes registers in the VCMP module
 *
 *************************************************************************/
void VCMP_Init(void)
{
	VCMP_TypeDef *vcmp = VCMP;
	
	vcmp->CTRL = _VCMP_CTRL_RESETVALUE;
	vcmp->CTRL |=	VCMP_CTRL_HYSTEN |
				VCMP_CTRL_WARMTIME_512CYCLES;
	
	vcmp->INPUTSEL = _VCMP_INPUTSEL_RESETVALUE;
	vcmp->INPUTSEL |=	VCMP_INPUTSEL_LPREF |
					(VCMP_INITIAL_LEVEL & _VCMP_INPUTSEL_TRIGLEVEL_MASK);
	
	vcmp->CTRL |= VCMP_CTRL_EN;
}
